c***********************************************************************
c     EHOOK
c***********************************************************************

      subroutine ehook(mw)
     
c     Created by : Jingzhi Pu, July 2000
c   
c     Description :
c           This routine optionally accepts the mass-weighted cartesian 
c       coordinates and transforms them to cartesian coordinates (mw=1).
c       The energy is computed using the cartesian coordinates.  If the
c       mass-weighted option was chosen (mw=1), the coordinates are
c       transformed back to mass-weighted at the end.  
c
c     Called by : ohook, 
c     Calls : mrest, trans, redinf, escener, effbath

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'                                           0718PJ00
      common /ncgcom/ iop
      dimension fmat(n3tm,n3tm)
     
c --- If it is a Grate restart, read the energy     
     
      irout=1
      if (irest.eq.1) call mrest(irout,dummy)


c --- If the information is read from restart file, no further calc     0816PJ00
c     needed, just return                                               0816PJ00 

      if (irout.eq.0) return


c --- Optionally convert the mass-weighted coordinates to cartesian

      if (mw.eq.1) call trans(2,n3,amass,x,dx)


c --- Compute the potential using the appropriate routines


c --- If the energy corresponds to a stationary point, check if it has
c     already been calculated and stored in redinf.

        if (iop.eq.6) then                                              1128JC97
            i=5
            nacat=natom
        else 
            if (iop.gt.0) then


c --- Read information of reactants, products, saddle point, etc        0725PJ00
c     for polyrate through subroutine redinf.
 

c --- iop = 1 or 2  ---> reactant 1 or 2                                0725PJ00
c         = 3 or 4  ---> product  3 or 4                                0725PJ00
c         = 5       ---> saddle point                                   0725PJ00
c         = 6 or 7  ---> reactant or product well                       0725PJ00
  
          call redinf(iop,fmat,ierr)
          if (ierr.eq.0) goto 1
            i=iop
          else
            i=-iop
            iop=-iop
          endif
            if (iop.eq.5) then
                nacat=natom
            else
                nacat=nratom(iop)
            endif
        endif


c --- If the point has not been stored, it calculates it

      ich=icharge(i)
      imul=imultipl(i)
      call escener(natoms,ich,imul,natom,label,
     *             iatsv,v,x,iop,nacat)


c --- Optionally transfer back to mass-weighted coordinates

1     if (mw.eq.1) call trans(1,n3,amass,x,dx)


c --- Put the energy at an appropriate zero of energy

      if (lezer0) v = v-ezer0     


c --- new format, the bath term is a function of X instead of S

      if (lbath) then
        call effbath(0)
      endif
    
c --- If it is a Grate restart, write the energy
     
      if (irest.eq.1) then
          irout=5
          call mrest(irout,dummy)
      endif

      return
      end                                           



c***********************************************************************
c     GHOOK
c***********************************************************************

      subroutine ghook(mw)

c     Created by : Jingzhi Pu, July 2000
c 
c     Description :
c          This routine optionally accepts the mass-weighted cartesian 
c       coordinates and transforms them to cartesian coordinates (mw=1).
c       The cartesian coordinates are used when computing the energy and
c       first derivatives of the energy with respect to each cartesian 
c       coordinate.  If the mass-weighted option was chosen (mw=1), the 
c       coordinates and first derivatives are transformed back to mass-
c       weighted at the end.  
c  
c     Called by :
c     Calls : trans, redinf, escfirst, effbath, mrest, 


      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'                                           0717PJ00
      common /ncgcom/ iop
      dimension fmat(n3tm,n3tm)
     
c --- If it is a Grate restart, read the energy and first der.     
     
      irout=2
      if (irest.eq.1) call mrest(irout,dummy)
      if (irout.eq.0) return

c --- Optionally convert the mass-weighted coordinates to cartesian

      if (mw.eq.1) call trans(2,n3,amass,x,dx)

c --- Compute the potential using the appropriate routines
c     If the calculation corresponds to a stationary point, check if it has
c     already been calculated and stored in redinf.

      if (iop.eq.6) then                                                1128JC97
          i=5
          nacat=natom
      else
          call redinf(iop,fmat,ierr)
          if (ierr.eq.0) goto 1
          i=iop
          if (iop.eq.5) then
              nacat=natom
          else
              nacat=nratom(iop)
          endif
      endif

c --- If the point has not been stored, it calculates it

      ich=icharge(i)
      imul=imultipl(i)
      call escfirst(natoms,ich,imul,natom,label,
     *              iatsv,v,x,dx,iop,nacat)

c --- Put the energy at an appropriate zero of energy

1     if (lezer0) v = v-ezer0

c --- calculate the derivative and energy, should be in Cartesian coord

      if (lbath) then
        call effbath(1)
      endif

c --- Transfer back to mass-weighted coordinates
      if (mw.eq.1) call trans(1,n3,amass,x,dx)


c --- If it is a Grate restart, write the energy and first der.

      if (irest.eq.1) then
          irout=6
          call mrest(irout,dummy)
      endif

      return
      end                                           



c***********************************************************************
c     OHOOK
c***********************************************************************

      subroutine ohook (iop)

c     Created by : Jingzhi Pu, July 2000
c     
c     Description :
c         This routine optionally accepts a initial guess to the cartesian
c       coordinates of the full system and returns the optimized geometry
c       and energy
c
c         If Multilevel is optimizing, all the optimization has been carried 
c       out in prep. Therefore, this subroutine only reads the and prints 
c       out the results of the calculations in prep
c
c       The geometry is optimized for a given chemical species:
c          for IOP = 1, or 2; the chemical species is reactant 1 or 2.
c          for IOP = 3, or 4; the chemical species is product 1 or 2.
c          for IOP = 5, the chemical species is the saddle point.
c          for IOP = 6, or 7; the chemical species is reactants or 
c                            products wells.
c     
c     Called by :
c     Calls : gsetp, newt, ehook, mrest, effbath 

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'
      dimension fmat(n3tm,n3tm)

c --- gsetp is a subroutine in energetics.f. It contains the preliminary 
c     geometry optimization calulations needed when POLYRATE is used with
c     an analyitcal surface.


     
c --- if Polyrate is going to do the optimization, call newt.

      if (igpot.eq.0) then
                 call gsetp(iop)
                 if (nratom(iop).gt.1) then
                    call newt(iop)
                 else 
                    call ehook(0)
                 endif
                 if (lezer0) v = v-ezer0
                 return
      endif
      if (isup.eq.1) call gsetp(iop)

c --- else:
c     If it is a Grate restart, read the geom, energy and 1st der.
     
      irout=4
      if (irest.eq.1) call mrest(irout,dummy)
      if (irout.eq.0) goto 3

      if (iop.le.2) then
           v=enresv
            if (isup.eq.1) then
                if (iop.eq.1) v=ereac1
                if (iop.eq.2) v=ereac2
            endif
           do m=1,n3tm
              x(m)=geresv(m)
             dx(m)=fsresv(m)
           enddo
      else if (iop.le.4) then
           v=enprsv
            if (isup.eq.1) then
                if (iop.eq.3) v=eprod1
                if (iop.eq.4) v=eprod2
            endif
           do m=1,n3tm
              x(m)=geprsv(m)
             dx(m)=fsprsv(m)
           enddo
      else if (iop.eq.7) then                                           1128JC97
           v=enwrsv                                                     1128JC97
           do m=1,n3tm                                                  1128JC97
              x(m)=gewrsv(m)                                            1128JC97
             dx(m)=fswrsv(m)                                            1128JC97
           enddo                                                        1128JC97
      else if (iop.eq.8) then                                           1128JC97
           v=enwpsv                                                     1128JC97
           do m=1,n3tm                                                  1128JC97
              x(m)=gewpsv(m)                                            1128JC97
             dx(m)=fswpsv(m)                                            1128JC97
           enddo                                                        1128JC97
      else
           v=enstsv
           do m=1,n3tm
              x(m)=gestsv(m)
             dx(m)=fsstsv(m)
           enddo
      endif
    
c --- Print out the results of the MULTILEVEL optimization

c --- Geometry

3     if (iop.eq.5) write(fu6,1006)
      write (fu6,1001)
      write (fu6,1011)
      inat=1
1     if (iop.ne.5) then
         inum = iatsv(inat,iop)
      else 
         inum = inat
      endif
         write(fu6,1021)inum,(x(3*(inum-1)+j), j=1,3)
      inat=inat+1
      if (iop.eq.5) then                                                1128JC97
           if (inat.le.natom) goto 1
      else
           if (iatsv(inat,iop).ne.0) goto 1
      endif
      write (fu6,1011)

c --- Gradients

      write (fu6,1031)
      write (fu6,1011)
      inat=1
2     if (iop.ne.5) then                                                0615JC98
         inum = iatsv(inat,iop)
      else
         inum = inat
      endif
         write(fu6,1021)inum,(dx(3*(inum-1)+j), j=1,3)
      inat=inat+1
      if (iop.eq.5) then                                                1128JC97
           if (inat.le.natom) goto 2
      else
           if (iatsv(inat,iop).ne.0) goto 2
      endif
      write (fu6,1011)

c --- calculate the derivative and energy, should be in Cartesian coord

      if (lbath) then
        call effbath(1)
      endif



c --- If it is a Grate restart, write the geom., energy and 1st der.
     
      if (irout.eq.0) return
      if (irest.eq.1) then
          irout=8
          call mrest(irout,dummy)
      endif
      return
1001  format(/4x,'Geometry optimized using Mutlilevel (a.u.)')
1006  format(//1X,32(1H*),' Saddle point ',32(1H*)//)
1011  format(/4x,55('-'),/)
1021  format(4x,i2,8x,3(f11.6,4x))
1031  format(/4x,'Gradients of the optimized structure (a.u.)')
      end                                           



c***********************************************************************
c     MINPENE   
c***********************************************************************

      subroutine minpene(ichargej,imultiplj,natom,label,iatsv,coord,
     *                   jtype,nacat,natesc)

c     Created : Jingzhi Pu, August 2000
c     Purpose : Write a standard Multilevel input file fu81 for single  0826PJ00
c               point energy calculation.                               0826PJ00
c               
c     Decription :
c           This subroutine writes the input for a Multilevel single    0826PJ00
c         point ennergy calculation. The current method to write such   0826PJ00 
c         a file is using the Saddle Point Multilevel input file  as    0826PJ00 
c         a template.      
c             1. parse off the Multilevel optimization part.            0826PJ00 
c             2. rewrite the geomtry and atomic lable                   0826PJ00
c             3. rewrite the number of atoms, the charge and            0826PJ00 
c                multiplicity.                                          0826PJ00
c             4. rewrite the single point energy option.                0826PJ00
c 
c         Q: how to realize the gradient, and hessian calculation       0826PJ00
c            for those stationary points? There's no gradient and       0826PJ00
c            hessian option in fu71 ~ fu75                              0826PJ00
c
c     Called by : escener 
c     Calls : mptoff, minpgeo, mtask                                    0826PJ00 

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'multirate.inc'
      dimension coord(n3tm),dx(n3tm)
      dimension label(natesc),iatsv(natesc,8)                           1128JC97
     

c --- set the single point energy calculation switch on                 0825PJ00

      mlene = 1
      mlgrad = 0
      mlhess = 0

c --- parsing of the optimization part                                  0824PJ00

      call moptoff                                                      0824PJ00


c --- write the atomic lable and geometry cartesian coordinates         0824PJ00

      call minpgeo(natom,label,iatsv,coord,jtype,nacat,natesc)          0824PJ00


c --- change the option to single point energy calculation and          0825PJ00
c     rewrite the number of atoms, charge and multiplicity              0825PJ00

      call mtask(mlene,mlgrad,mlhess,ichargej,imultiplj,
     *           natom,jtype,nacat)                                     0825PJ00


c --- The energy keyword (enerot), %0 option (cerblk),                  0826PJ00
c     non-standard basis set(baslk) writing are removed.                0826PJ00


c --- write done

      return


      end


c***********************************************************************
c     MINPFRS
c***********************************************************************

      subroutine minpfrs(ichargej,imultiplj,natom,label,iatsv,coord,
     *                   jtype,nacat,natesc)

c     Created : Jingzhi Pu, August 2000
c 
c     Decription :
c
c          Subroutine that writes the appropriate Mutilevel input file
c       for a first derivative calculation
c
c     Called by : escfirst,
c     Calls : moptoff, minpgeo, mtask

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'multirate.inc'
      dimension coord(n3tm),dx(n3tm)
      dimension label(natesc),iatsv(natesc,8)                           1128JC97

c --- set the single point derivative calculation switch on             0825PJ00

      mlene = 0
      mlgrad = 1
      mlhess = 0

c --- parsing of the optimization part                                  0824PJ00

      call moptoff                                                      0824PJ00


c --- write the atomic lable and geometry cartesian coordinates         0824PJ00

      call minpgeo(natom,label,iatsv,coord,jtype,nacat,natesc)          0824PJ00


c --- change the option to single point energy calculation and          0825PJ00
c     rewrite the number of atoms, charge and multiplicity              0825PJ00

      call mtask(mlene,mlgrad,mlhess,ichargej,imultiplj,
     *           natoma,jtype,nacat)                                    0825PJ00


c --- write done

      return

      end



c***********************************************************************
c     MINPGEO
c***********************************************************************

      subroutine minpgeo(natom,label,iatsv,coord,jtype,nacat,natesc)

c     Greated : Jingzhi Pu, Aug 2000
c     Purpose : read every line and skip certain part of
c               the input file
c
c     Description : this is a test for the multirate input
c                   file. The goal is to read the input file
c                   according the type required by the caller,
c                   then read the input file line by line
c                   and write it into fu81. If it encounters
c                   the geometry part,  reading the old geom
c                   line but don't write it in to new input.
c                   As instead, write the new geomtry into
c                   the new multilevel input file .

      implicit double precision (a-h, o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'multirate.inc'
      dimension coord(n3tm),dx(n3tm)
      dimension label(natesc),iatsv(natesc,8)                           

      character*80 strread
      character*80 case
      character*80 tempstr
      character*5 geoid
      character*3 endid
      character*2 asymb(103)
      character*2 atmlb

     
      external case

c --- set the atomic lable, because Multilevel doesn't support          0829PJ00   
c     the atomic number as input, we have to convert the atomic         0829PJ00 
c     number to atomic lable                                            0829PJ00

      data (asymb(i),i=1,103)  /                                        0829PJ00  
     *   'H ','He','Li','Be','B ','C ','N ','O ','F ','Ne',
     *   'Na','Mg','Al','Si','P ','S ','Cl','Ar',
     *   'K ','Ca','Sc','Ti','V ','Cr','Mn','Fe','Co','Ni','Cu',
     *        'Zn','Ga','Ge','As','Se','Br','Kr',
     *   'Rb','Sr','Y ','Zr','Nb','Mo','Tc','Ru','Rh','Pd','Ag',
     *        'Cd','In','Sn','Sb','Te','I ','Xe',
     *   'Cs','Ba','La','Ce','Pr','Nd','Pm','Sm','Eu','Gd','Tb',
     *        'Dy','Ho','Er','Tm','Yb','Lu','Hf','Ta','W ','Re',
     *        'Os','Ir','Pt','Au','Hg','Tl','Pb','Bi','Po','At','Rn',
     *   'Fr','Ra','Ac','Th','Pa','U','Np','Pu','Am','Cm','Bk',
     *        'Cf','Es','Fm','Md','No','Lr'/
 
      mgeobeg = 0
      mgeoend = 0

      geoid = 'geom '
      endid = 'end'


c --- Open the old and new input files.

c --- fu86 is a Multilevel input template copied from fu75              0829PJ00      

      open(unit=fu86, status='unknown', file='esp.fu86')
      open(unit=fu81, status='old', file='esp.fu81')


c --- read one line from input fu81.


1     read(fu81, 100, end=200)strread

c --- covert the line to lowerc case, so that it will be compatable to  0825PJ00     
c     either the upper case or lower case Multilevel input file         0825PJ00

      tempstr = strread
      strread = case(strread)


c --- if the before geometry section, we need to check geoid

      if (mgeobeg .eq. 0) then

          do 120 ilinpos=1,5

              if (strread(ilinpos:ilinpos+4).ne.geoid) then             0825PJ00 
                  goto 120
              else
                  mgeobeg = 1
              endif

120       continue

c --- write this line either it's before geoid or it's geoid

          write(fu86, 100) tempstr

c --- if geomid has be read, check the geometry end.

      else if (mgeoend .eq. 0) then

c --- now we are within the geometry section

          do 140 ilinpos=1,5

              if (strread(ilinpos:ilinpos+2) .ne. endid) then           0825PJ00 
                  goto 140
              else
                  mgeoend = 1
              endif

140       continue

c --- if it's not a geometry end, we skip it, otherwise write file

           if (mgeoend .eq. 1) then

c --- first, write the new geometry

               i=1

c --- if it is not saddle point or Generalized Transition State         

10             if (jtype.ne.5.and.jtype.ne.6) then       
                   inum = iatsv(i,jtype)
               else
                   inum = i
               end if

               ilab = label(inum)
               atmlb = asymb(ilab)                                      0829PJ00  
               write(fu86,1021)atmlb,(coord(3*(inum-1)+j), j=1,3)       0829PJ00 

               i=i+1
         
               if (jtype.eq.5.or.jtype.eq.6) then                                
                   if (i.le.natom) then
                        goto 10
                   endif
               else
                   if (iatsv(i,jtype).ne.0) then
                        goto 10
                   endif

               endif

c --- Then write the 'END' keyword for geometry section

               write(fu86, 100) tempstr 

           endif

c --- if both geometry begin and end are read, just write the line

      else
2             write(fu86, 100) tempstr
      endif

      goto 1


200   close(fu86)
      close(fu81)

      return

100   format(a80)
1021  format (6x,a3,t20,d15.8,t40,d15.8,t60,d15.8)

      end



c***********************************************************************
c     MINPSEC
c***********************************************************************

      subroutine minpsec(ichargej,imultiplj,natom,label,iatsv,coord,
     *                   jtype,nacat,natesc)

c     Created by : Jingzhi Pu, August 2000
c     
c     Description :
c         Subroutine that writes the appropriate MULTILEVEL input file
c       for performing a second derivative calculation
c
c     Called by : escsec, secin 
c     Calls :

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'multirate.inc'

      dimension coord(n3tm),dx(n3tm)
      dimension label(natesc),iatsv(natesc,8)                           1128JC97

c --- set the single point hessian calculation switch on                0825PJ00

      mlene = 0
      mlgrad = 0
      mlhess = 1


c --- parsing of the optimization part                                  0824PJ00

      call moptoff                                                      0824PJ00


c --- write the atomic lable and geometry cartesian coordinates         0824PJ00

      call minpgeo(natom,label,iatsv,coord,jtype,nacat,natesc)          0824PJ00


c --- change the option to single point energy calculation and          0825PJ00
c     rewrite the number of atoms, charge and multiplicity              0825PJ00

      call mtask(mlene,mlgrad,mlhess,ichargej,imultiplj,
     *           natom,jtype,nacat)                                     0825PJ00


c --- The hessian keyword (secrout), %0 option (cerblk),                0826PJ00
c     non-standard basis set(baslk) writing are removed.                0826PJ00


c --- write done


      return


      end




c***********************************************************************
c     MOPTOFF   
c***********************************************************************

      subroutine moptoff

c     Greated : Jingzhi Pu, August 2000
c     Purpose : read every line and skip certain part of
c               the input file
c
c     Description : this is a test for the multirate input
c                   file. The goal is to read the input file
c                   according the type required by the caller,
c                   then read the input file line by line
c                   and write it into fu81. If it encounters
c                   the geometry part,  reading the old geom
c                   line but don't write it in to new input.
c                   As instead, write the new geomtry into
c                   the new multilevel input file .

      implicit double precision (a-h, o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'

      character*80 strread
      character*80 case
      character*80 tempstr
      character*9 moptid
      character*7 eoptid                                                0915PJ00 
      character*1 secid

      external case

      moptbeg = 0
      newsec = 0

c --- eoptid is added here for the external optimization part in the    0915PJ00
c     multilbevel input file                                            0915PJ00

      moptid = '*multiopt'
      eoptid = '*extopt'
      secid = '*'


c --- Open the old and new input files.

c --- fu86 is a template Multilevel input file copied from fu75         0829PJ00                

      open(unit=fu86, status='old', file='esp.fu86')
      open(unit=fu81, status='unknown', file='esp.fu81')


c --- read one line from input fu86.


1     read(fu86, 100, end=200)strread


c --- convert the line to lower case, so that it will be compatable     0825PJ00  
c     to either the upper case or lower case Multilevel input file      0825PJ00

      tempstr = strread
      strread = case(strread)


c --- if the before geometry section, we need to check geoid

      if (moptbeg .eq. 0) then

          do 120 ilinpos=1,5

              if ( (strread(ilinpos:ilinpos+8) .ne. moptid) 
     *                         .and.
     *             (strread(ilinpos:ilinpos+6) .ne. eoptid) ) then      0915PJ00 
                 goto 120
              else
                  moptbeg = 1
              endif

120       continue

c --- write if it's not opt section, otherwise continue read

          if (moptbeg .eq. 0) then
              write(fu81, 100) tempstr
          endif

c --- if geomid has be read, check the geometry end.

      else if (newsec .eq. 0) then

c --- now we are within the geometry section

          do 140 ilinpos=1,5

              if (strread(ilinpos:ilinpos) .ne. secid) then
                  goto 140
              else
                  newsec = 1
              endif

140       continue

c --- if it's not a geometry end, we skip it, otherwise write file

           if (newsec .eq. 1) then
              write(fu81, 100) tempstr
           endif

c --- if both geometry begin and end are read, just write the line

      else
2             write(fu81, 100) tempstr 
      endif

      goto 1


200   close(fu86)
      close(fu81)

      return

100   format(a80)


      end

c***********************************************************************
c     MOUTSEC
c***********************************************************************

      subroutine moutsec (iop,v,coord,tmpdx,fmat,jtype,iatsv,
     *                    natom,nacat,natesc)

c     Created by : Jingzhi Pu, July 2000
c     Purpose : Extract electronic structure information from Multilevel
c               output file.
c
c     Description :
c        Subroutine that extracts the Geometry, Energy, Gradient and    0816PJ00
c      Hessian information form the Multilevel formatted summary file   0816PJ00
c      ml.sum                                                           0816PJ00
c        The summary file is created by the c shell script 'mlsum'      0816PJ00
c     so the .ml script is needed to be modified so that every time     0816PJ00
c     the ml.sum file is created along the ml.out file                  0816PJ00
c
c
c     Called by : enatin, geoin, secin, escener, escfirst,    
c     Calls : case, cfloat, 

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'multirate.inc'


      dimension coord(n3tm),dx(n3tm),fmat(n3tm,n3tm)
      dimension tmpfmat(n3tm,n3tm), iatsv(natesc,8)                     1128JC97
      dimension tmpfm(n3tm*n3tm), tmpdx(n3tm)
      character*80 strread
      character*80 case
      character*25 secid
      character*16 forceid
      character*12 eneid
      character*28 geomid
      character*1 charkey(10)

c     iop = 0 obtain energy
c     iop = 1 obtain energy and geometry
c     iop = 2 obtain energy, geometry, and gradient
c     iop = 3 obtain energy, geometry, gradient, and hessian



c --- First, initial the dirivative vector and the force matrix 
      
      do i=1, n3tm
        tmpdx(i)=0
      enddo
      do i=1, n3tm*n3tm
        tmpfm(i)=0
      enddo

c --- The Multileve summary file ml.sum is created to record all        0810PJ00 
c     energy, gradient and hassian, this file is similar to the         0810PJ00
c     Gaussian checkpoint file Test.Fchk

c --- It starts looking for the energy by means of the title
c     'total energy'.

      open (unit=fu85,status='unknown',file='ml.sum')                   0810PJ00

c --- When enounter the end of file unit fu85, go to 1002,etc           0725PJ00
c     (explanation of the end = 1002 part)                              0725PJ00
c
c     Because Mutilevel extract a summary file called ml.sum to record  0816PJ00
c     the number of Atoms, the Charge, the Multiplicity, the Geometry   0816PJ00
c     the Gradient and the Hessian, so Multirate read this summary file 0816PJ00
c     and look for the information it needs.                            0816PJ00


c --- Modification needed here, for the ml.sum doesn't marker           0816PJ00
c     geometry this way. And only the geometry in a.u. is needed        0816PJ00



c --- Now, it will look for the cartesian coordinates, by means
c     of the title 'atomic coordinates (in a.u.)'

      geomid='atomic coordinates (in a.u.)'                             0819PJ00

c --- if it's not a geometry optimization, jump to energy reading       0826PJ00

      if (iop.lt.1) goto 1015                                           0826PJ00

6015  read (fu85,1041,end=1003) strread
      do 6120 ilinpos=1,6
          strread=case(strread)
          if (strread(ilinpos:ilinpos+27).ne.geomid) then               0819PJ00
                    goto 6120
             else

 
c --- read off the title lines under the geomid                         0819PJ00

                 do i = 1, 3                                            0819PJ00
                     read(fu85, 1041, end = 1003)strread                0819PJ00
                 enddo                                                  0819PJ00

                 i=0
                 numdxn=nacat*3

c --- The main difference is the Gaussian checkpoint file write         0816PJ00
c     geometry in 5 number/line, but ml.sum is 3 number/line            0816PJ00


c --- numdxn here is the number of coordinates. So in the ml.sum        0725PJ00
c     there are two cases: 1. the number of coordinates is less than    0725PJ00
c     3, so this subroutin read for coordinate directly. 2. if the      0725PJ00
c     number of coordinates is more than 3, read 3 number each time     0725PJ00
c     beacuse the checkpoint file write 3 coordinates in one line.      0725PJ00
c     Then decrease numdxn by 3, continue to read until numxn become    0725PJ00
c     less than 3.                                                      0725PJ00


6035             if (numdxn.gt.3) then                                  0816PJ00


c --- checkpoint file write 3 coordinates in one line as a group.       0725PJ00

                    read (fu85,1473) (coord(i+j), j=1,3)                0819PJ00
                    numdxn=numdxn-3                                     0816PJ00
                    i=i+3
                    goto 6035
                 else

c --- numdxn here is equal to nacat * 3                                 0725PJ00


                    read (fu85,1473) (coord(i+j), j=1,numdxn)           0819PJ00
                 endif
                    goto 1015                                           0819PJ00
          endif
6120  continue
      goto 6015


1002  write (fu6,*) 'Error reading Mutlilevel summary file - 1'         0816PJ00
      stop
1003  write (fu6,*) 'Error reading Multilevel summary file - 2'         0816PJ00
      stop
1004  write (fu6,*) 'Error reading Multilevel summary file - 3'         0816PJ00
      stop
1005  write (fu6,*) 'Error reading Multilevel summary file - 4'         0816PJ00
      stop


c --- For the total energy part, Multirate recognize the total energy   0725PJ00
c     by searching  the checkpoint file for the keyword of 'total       0725PJ00
c     energy'.                                                          0725PJ00

1015  eneid='total energy'                                              0826PJ00

      read (fu85,1041,end=1002) strread
      do 120 ilinpos=1,6
          strread=case(strread)
          if (strread(ilinpos:ilinpos+11).ne.eneid) then
                    goto 120
          else


c --- cfloat is a function in polyrate's file interface.f.              0725PJ00

c     The function use a string varible as parameter, which is this     0725PJ00
c     case is an energy number grabbed from the gaussian checkpoint     0725PJ00
c     file. It strippes the blank part in front of the string, and      0725PJ00
c     then get the part before decimal point and the part after de-     0725PJ00
c     cimal point as well as the exponent. The final effect of this     0725PJ00
c     function is conver the grabbed string to numerical varible.       0725PJ00



c --- use ilinpos+12 to point the next character after the eneid        0816PJ00
   
                    v=cfloat(strread(ilinpos+12:80))                    0816PJ00
                    goto 1025
          endif
120   continue
      goto 1015



c --- Now, it will look for the cartesian forces, by means
c     of the title 'Cartesian forces' 



c --- remove varible 'frcide1' here, because in gaussrate, there is     0816PJ00
c    a version problem -- for different version of gaussian the         0816PJ00
c    checkpoint file uses different gradient title, but for             0816PJ00 
c    Multilevel, there's no such a problem                              0816PJ00

      
c ---The corresponding modification is made in the varible              0816PJ00
c    declaration part.                                                  0816PJ00


1025  forceid='cartesian gradie'
      if (iop.lt.2) goto 1999

2015  read (fu85,1041,end=1004) strread
      do 220 ilinpos=1,6
          strread=case(strread)
          if (strread(ilinpos:ilinpos+15).ne.forceid) then              0820JC97
                 goto 220
          else
                 i=0
                 numdxn=nacat*3


c --- The corresponding modification from 5 --> 3                       0816PJ00
   

2025             if (numdxn.gt.3) then                                  0816PJ00 
                    read (fu85,1473) (tmpdx(i+j), j=1,3)                0819PJ00 
                    numdxn=numdxn-3                                     0816PJ00
                    i=i+3                                               0816PJ00
                    goto 2025                               
                 else                                      
                    read (fu85,1473) (tmpdx(i+j), j=1,numdxn)           0819PJ00
                 endif
                 goto 2035
          endif
220   continue
      goto 2015


c --- Now, it will look for the second derivative information
c     by means of the title 'Cartesian forces constants' 

2035  secid='cartesian force constants'
3015  read (fu85,1041,end=1005) strread
      if (iop.lt.3) goto 1999
      do 320 ilinpos=1,6
          strread=case(strread)
          if (strread(ilinpos:ilinpos+24).ne.secid) then
                 goto 320
          else
                 i=0

c ---  Corresponding modification here 5 --> 3                          0816PJ00


                 numdxn=((nacat*3)*(nacat*3-1))/2+nacat*3
3025             if (numdxn.gt.3) then
                    read (fu85,1473) (tmpfm(i+j), j=1,3)                0819PJ00
                    numdxn=numdxn-3
                    i=i+3
                    goto 3025
                 else
                    read (fu85,1473) (tmpfm(i+j), j=1,numdxn)           0819PJ00
                 endif
                 goto 3035
          endif
320   continue
      goto 3015

c --- Now we need a reordering of the second derivative information
c     We will convert the vector of triangular force constants, TMPFM,
c     into the hessian matrix TMPFMAT

3035  k=1
      do 410 inx=1,3*nacat
        do 420 iny=1,inx

c --- Initialize the triangle matrix, and than get the symmetric        0725PJ00
c     element in the other triangle part by transformation              0725PJ00
c     part by transformation.                                           0725PJ00
 
         tmpfmat(inx,iny)=tmpfm(k)
         if (inx.ne.iny) tmpfmat(iny,inx)=tmpfmat(inx,iny)
         k=k+1
420     continue
410   continue

c --- use jtype to check whether it is the saddle point hessian,        0725PJ00
c     if not, needed to be reordered.


      if (jtype.ne.5.and.jtype.ne.6) then                               1128JC97

c --- Now, according to the ordering of the atoms at the saddle point,
c     we reorder the hessian information

         do l=1,nacat
         do m=1,nacat
           do i=1,3
           do j=1,3
             innx=3*(iatsv(l,jtype)-1)+i
             inny=3*(iatsv(m,jtype)-1)+j
             inox=3*(l-1)+i
             inoy=3*(m-1)+j
               fmat(innx,inny)=tmpfmat(inox,inoy)
           enddo
           enddo
         enddo
         enddo
      else

c --- If we have calculated the saddle point Hessian, we don't need the
c     reordering

         do innx=1,n3tm
         do inny=1,n3tm
             fmat(innx,inny)=tmpfmat(innx,inny)
         enddo
         enddo
      endif

1999  close (fu85)

      return


c --- use format reading to parse off the atomic label, so the          0819PJ00
c     first 20 characterin the line is ignored.                         0819PJ00 

1473  format(t20,d15.8,t40,d15.8,t60,d15.8)                             0819PJ00 
1041  format (a80)                                                     
      end




c***********************************************************************
c     MREST
c***********************************************************************

      subroutine mrest(irout,fmat)

c     Created by : Jingzhi Pu, July 2000
c 
c     Description :
c        This subroutine compares the geometry with the one in the file and
c      if they are the same, it reads the properties of the point, so that
c      it will not be calculated again by means of Multilevel.
c      If the properties were stored, it returns irout=0
c 
c     Called by : enatin, ehook, ghook, ohook
c     Calls :

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'
      common /ncgcom/ iop
      common /nccmcm/ iopc
      dimension fmat(n3tm,n3tm)
      fallout=1.0d-8
      icontrw=0

c --- The firt thing we have to do is tp check if we only want to write
c     (then, irout>4) or we want reading and writing. In the last case:


1     if (irout.le.4) then


c --- First, if the previous comparison was ok, we read everything and put 
c     it on an array for the next comparisons

         read (fu83,*,end=999) irread                                    

c --- IP is the calculation status mark in the restart file fu83        0820PJ00  
c                   1 ~ 4 reading :               5 ~ 8 writing:        0820PJ00
c        Energy        (1)                           (5)                0820PJ00
c        Gradient      (2)                           (6)                0820PJ00
c        Hessian       (3)                           (7)                0820PJ00
c        Geometry      (4)                           (8)                0820PJ00


        if (irread.eq.4) read(fu83,*,end=999)ip,icharge(ip),
     *                         imultipl(ip)

c --- Read Energy

        read (fu83,*,end=999) vread


        do icount=1,3*natom
        read (fu83,*,end=999) georead(icount)
        enddo
        do icount=1,3*natom
        read (fu83,*,end=999) dxread(icount)
        enddo
        if (irread.eq.3) then
        do icount=1,3*natom
        do jcount=1,3*natom
          read (fu83,*,end=999) fmread(icount,jcount)
        enddo
        enddo
      endif

c --- If the stored point does the same calculation that we want to do,
c     continue. Otherwise, the program looks for a place where it is done

        if (irout.ne.irread) then
               if (icontrw.eq.0) then
                  icontrw=1
                  rewind fu83
                  goto 1
               else
                  goto 1
               endif
        endif

c --- If it is a geometry optimization, it reads geometry, energy and 1st der.

        if (irout.eq.4) then
          v=vread
          do icount=1,natom*3
             x(icount)=georead(icount)
          enddo
          do icount=1,natom*3
             dx(icount)=dxread(icount)
          enddo

c --- and now writes is to gwrite

          write (fu84,*) irread
          if (irread.eq.4) write(fu84,*)ip,icharge(ip),imultipl(ip)
          write (fu84,*) v
          do icount=1,3*natom
          write (fu84,*) x(icount)
          enddo
          do icount=1,3*natom
          write (fu84,*) dx(icount)
          enddo
   
c --- and sets irout=0 to indicate that everything was ok.
       
          irout=0
          return
        else

c --- Now, if it wasn't a geometry optimization, compare the geometres. 
c     If the one to be used in the calculations is different from the one in 
c     gwrite, it returns (it doesn't allow rotations or translations in order
c     to avoid problems along the reaction path). If IOP<5 (i.e., before
c     enetring the reaction path calculations) the comparison between
c     geometries is not performed. It is assumed that for reactants,
c     products and saddle point everything is exactly the same than
c     in the original calculation
c     The same for wells (IOP>6)

          if (iop.eq.6) then                                            1128JC97
          do icount=1,natom*3
            diff=x(icount)-georead(icount)
            if (abs(diff).gt.fallout) then  
               if (icontrw.eq.0) then
                  icontrw=1
                  rewind fu83
                  goto 1
               else
                  goto 1
               endif
            endif
          enddo
          endif
          
c --- If both geometries are the same, it reads the stored information
c     and writes to gwrite

          v=vread
          if (irout.gt.1) then
            do icount=1,natom*3
               dx(icount)=dxread(icount)
            enddo
          endif
          if (irout.eq.3) then
            do icount=1,natom*3
             do jcount=1,natom*3
               fmat(icount,jcount)=fmread(icount,jcount)
             enddo
            enddo
          endif
        write (fu84,*) irread
        write (fu84,*) vread
        do icount=1,3*natom
        write (fu84,*) georead(icount)
        enddo
        do icount=1,3*natom
        write (fu84,*) dxread(icount)
        enddo
        if (irout.eq.3) then
        do icount=1,3*natom
        do jcount=1,3*natom
         write (fu84,*) fmread(icount,jcount)
        enddo
        enddo
        endif
   
c --- and sets irout=0 to indicate that everything was ok.
       
          irout=0
      endif
      return

c --- if irout>4 (i.e., we only want writing):

      else
          irout=irout-4
          write (fu84,*) irout
          if (irout.eq.4) write(fu84,*)iop,icharge(iop),imultipl(iop)
          write (fu84,*) v
          do icount=1,3*natom
               write (fu84,*) x(icount)
          enddo
          do icount=1,3*natom
               write (fu84,*) dx(icount)
          enddo
          if (irout.eq.3) then
             do icount=1,3*natom
                do jcount=1,3*natom
                   write (fu84,*) fmat(icount,jcount)
                enddo
             enddo
           endif
      endif
999   if (icontrw.eq.0) then
         icontrw=1
         rewind fu83
         goto 1
      else
         return
      endif
      return
      end



c************************************************************************
c     MTASK
c***********************************************************************

      subroutine mtask(mlene,mlgrad,mlhess,ichargej,imultiplj,
     *                 natom,jtype,nacat)                                         

c     Greated by : Jingzhi Pu, Aug 2000
c     Purpose : read every line and skip certain part of
c               the input file
c
c     Description : this is a test for the multirate input
c                   file. The goal is to read the input file
c                   according the type required by the caller,
c                   then read the input file line by line
c                   and write it into fu81. If it encounters
c                   the geometry part,  reading the old geom
c                   line but don't write it in to new input.
c                   As instead, write the new geomtry into
c                   the new multilevel input file .

      implicit double precision (a-h, o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'multirate.inc'

      character*80 strread
      character*80 case
      character*80 tempstr

      character*6 atomid
      character*6 chargid
      character*12 mulid
      character*8 unitid

      character*6 meneon
      character*8 meneoff

      character*8 mgradon
      character*10 mgradoff

      character*7 mhesson
      character*9 mhessoff

      external case

      atomid = 'natoms'
      chargid = 'charge'
      mulid = 'multiplicity'
      unitid = 'geomunit'

      meneon = 'energy'
      meneoff = 'noenergy'

      mgradon = 'gradient'
      mgradoff = 'nogradient'

      mhesson = 'hessian'
      mhessoff = 'nohessian'


c --- Open the old and new input files.

      open(unit=fu86, status='old', file='esp.fu86')
      open(unit=fu81, status='unknown', file='esp.fu81')



c --- read one line from input fu70.

1     read(fu86, 100, end=200)strread

c --- convert the line to lower case, so it will compatable to both     0825PJ00               
c     upper case and lower case Multilevel input file                   0825PJ00

      tempstr = strread
      strread = case (strread)


c --- rewrite number of atoms

          do 120 ilinpos=1,6

              if (strread(ilinpos:ilinpos+5) .ne. atomid) then
                  goto 120
              else

c --- nacat is the actual number of atoms                               0829PJ00 
 
                  write(fu81,300) nacat
                  goto 1
              endif

120       continue


c --- rewrite charge 

          do 125 ilinpos=1,6

              if (strread(ilinpos:ilinpos+5) .ne. chargid) then
                  goto 125
              else
                  write(fu81,400) ichargej
                  goto 1
              endif

125       continue


c --- rewrite multiplicity

          do 130 ilinpos=1,6

              if (strread(ilinpos:ilinpos+11) .ne. mulid) then
                  goto 130
              else
                  write(fu81,500) imultiplj
                  goto 1
              endif

130       continue


c --- rewrite the geometry unit

          do 140 ilinpos=1,6
   
              if (strread(ilinpos:ilinpos+7) .ne. unitid) then 
                  goto 140
              else 
                  write(fu81,550)
                  goto 1
              endif 

140       continue



c --- rewrite the energy switch option

          do 150 ilinpos=1,6

              if ((strread(ilinpos:ilinpos+5) .ne. meneon)
     *                             .and.
     *            (strread(ilinpos:ilinpos+7)  .ne. meneoff)) then
                  goto 150
              else if (mlene .eq. 1) then
                  write(fu81,600)
                  goto 1
              else 
                  write(fu81,650)
                  goto 1
              endif

150       continue


c --- rewrite the gradient switch option

          do 160 ilinpos=1,6

              if ((strread(ilinpos:ilinpos+7) .ne. mgradon)
     *                             .and.
     *            (strread(ilinpos:ilinpos+9) .ne. mgradoff)) then
                  goto 160
              else if (mlgrad .eq. 1) then
                  write(fu81,700)
                  goto 1
              else 
                  write(fu81,750)
                  goto 1
              endif

160       continue


c --- rewrite the hessian switch option

          do 170 ilinpos=1,6

              if ((strread(ilinpos:ilinpos+6) .ne. mhesson)
     *                             .and.
     *           (strread(ilinpos:ilinpos+8) .ne. mhessoff)) then
                  goto 170
              else if (mlhess .eq. 1) then
                  write(fu81,800)
                  goto 1
              else 
                  write(fu81,850)
                  goto 1
              endif

170       continue


c --- for the part don't need to be modified, just write.

      write(fu81,100)tempstr
      goto 1

200   close(fu86)
      close(fu81)
    
      return

100   format(a80)
300   format(4x,'NATOMS',i4)
400   format(4x,'CHARGE',i4)
500   format(4x,'MULTIPLICITY',i4)
550   format(4x,'GEOMUNIT',2x,'au')                                    
600   format(4x,'ENERGY')
650   format(4x,'NOENERGY')
700   format(4x,'GRADIENT')
750   format(4x,'NOGRADIENT')
800   format(4x,'HESSIAN')
850   format(4x,'NOHESSIAN')

      end


c***********************************************************************
c     PREP
c***********************************************************************
      subroutine prep
     
c     Created by : Jingzhi Pu, July 2000
c
c     Description :
c
c        This routine is the driver for all preparation routines needed 
c      to set up a multirate calculation.
c
c        This routine is called after all POLYRATE input has been read 
c      in and before the table of input is written to fu6.  It is also
c      before any calculations are done.  This routine is used to call
c      all electronic structure preparation routines. 
c
c      Comment: 
c             In fact, fu71 ~ fu75 are set to optimization and  energy  0826PJ00
c        calc. Prep calls the geoin to run shuttl_mr then get both the  0826PJ00
c        geometry and the energy. If the dynamic part needs hessian,    0826PJ00
c        secin in is called for rewrite the input esp.fu81, but with    0826PJ00
c        out the optimization keyword, just do the single point hessian 0826PJ00 
c        calc. If one of the stationary points is atom, no need to      0826PJ00
c        call secin, so enatin is used as substitution to run the       0826PJ00
c        the shuttl_mr script and get out the energy.                   0826PJ00  
c
c     Called by :
c     Calls : setup, geoin, rchmul, secin, enatin

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'
      common /ncgcom/ iop
      common /nccmcm/ iopc
      common /eshscm/ fmat(n3tm,n3tm)
      dimension xcm(3)
      dimension nrs(8)

c --- Check if it is the first call to this subroutine

      if (numci.eq.0) then

c --- if it is, set up some variables and start the Multilevel 
c     optimization procedure (if selected) and the hessian 
c     calculations for reactant and products.

      ereac=0.d0
      ereac1=0.d0
      ereac2=0.d0
      eprod=0.d0
      eprod1=0.d0
      eprod2=0.d0
      do i=1,8
        nrs(i)=nratom(i)
      enddo
      nrs(5)=natom
      do i=1,8
        vjdep(i)=0.D0
        icharge(i)=0
        imultipl(i)=1
        ioptgr(i)=0
      enddo
      call setup

c --- Set isavect. If it is set to zero, the computer time is reduced as
c     much as possible, but it can be dangerous for optimizing geometries,
c     calculating anharmonicities, and second (or higher) derivatives by 
c     means of Polyrate methods. If it is set to one, some calculations 
c     may be repeated, but it will be much safer for the above mentioned
c     calculations. If isavect=2, the calculation is completely safe,
c     no check for saving time is done and derivatives will be calculated
c     correctly.
c     isavect=1 has been removed, since it saves little computer time
c     and there is a certain risk of failure

      isavect=0
      do 3 inesp=1,8                                                    1128JC97
       if (inesp.eq.6) goto 3                                           1128JC97
       if (ispot.eq.0) isavect=3
       if (igpot.eq.0) then
        if (istatu(inesp).eq.0) then                                    0729YC97
               if (nrs(inesp).gt.1) then
                     isavect=2
                     goto 1
               endif
        endif
       endif
        if (iharm(inesp).ne.1) then
            do inm=1,nxmod(inesp)
             if (vharmr(inm,inesp).ne.'harmonic '.and.
     *           vharmr(inm,inesp).ne.'hinrot   ') then
                       isavect=3
             endif
            enddo
        endif
3     continue


1     numci=1 
      if (isavect.eq.2) then
             if (isup.eq.1) return
             if (isup.eq.0) then
                    write (fu6,1001)
                    stop 'prep 1'
             endif
      endif
      if (isavect.eq.3) then
             if (isup.eq.1) isavect=2
             if (isup.eq.0) then
                    write (fu6,1001)
                    stop 'prep 2'
             endif
      endif

c --- Calculations for reactants

      if (istatu(1).ne.6.or.istatu(2).ne.6) then                        0725YC97
      do iopc = 1,2                                          
        if (irepr(iopc).ne.0) then
            if (istatu(iopc).ne.6) then                                 0729YC97
            if (nratom(iopc).gt.1) then

c --- If the geometry is not optimized, carry out Multilevel            0809PJ00
c     geometry optimization by call geoin.                              0809PJ00

              if (ioptgr(iopc).eq.1.and.istatu(iopc).lt.2) then         0725YC97
                 call geoin(iopc,ircm)
                 if (ircm.eq.1) call rchmul(iopc)
              else
                 imultipl(iopc)=nedeg(3*iopc-2)                         0725YC97
                 do i=1,nratom(iopc)                                    0729YC97
                     inum = iatsv(i,iopc)                               0729YC97
                     do j = 2,0,-1                                      0729YC97
                        x(inum*3-j) = xr(inum*3-j,1)                    0729YC97
                     enddo                                              0729YC97
                 enddo                                                  0729YC97
              endif
                 if (isavect.ne.2) then
                   call secin(iopc)
                 else
                   itmp=iop
                   iop=iopc
                   call ehook(0)
                   iop=itmp
                 endif
            else

c --- For atomic species, it calls enatin instead of                    0729PJ00
c     geoin and secin.                                                  0729PJ00

              if (ioptgr(iopc).eq.1) then
                 call enatin(iopc,ircm)
                 if (ircm.eq.1) call rchmul(iopc)
              else
                 itmp=iop
                 iop=-iopc
                 call ehook(0)
                 iop=itmp
              endif
            endif
            else                                                        0729YC97
              v = steng(iopc)                                           0729YC97
              do i=1,nratom(iopc)                                       0729YC97
              inum = iatsv(i,iopc)                                      0729YC97
                 do j = 2,0,-1                                          0729YC97
                    x(inum*3-j) = xr(inum*3-j,1)                        0729YC97
                 enddo                                                  0729YC97
              enddo                                                     0729YC97
            endif                                                       0729YC97
c ---   vjdep(iopc)=v
        ereac=ereac+V
        if (isup.eq.1) then
            if (iopc.eq.1) then                                         1201JC97
               ereac1=v
            else
               ereac2=v
            endif
        endif
        endif
      enddo

c --- saves the result for the following calculations on these points

        if (isup.eq.0) then
            itmp=iop
            iop=5
            call ehook(0)
            iop=itmp
            ereac=v
        endif
       enresv=ereac
       if (isup.eq.0) ezer0=ereac
       do i=1,n3tm
       geresv(i)=x(i)
       xr(i,1)=x(i)
       x(i)=0.d0
       fsresv(i)=dX(i)
       dx(i)=0.d0
        do j=1,n3tm
        hsresv(i,j)=fmat(i,j)
        fmat(i,j)=0.d0
        enddo
       enddo
      endif                                                             0725YC97

c --- Calculations for products

      if (istatu(3).ne.6.or.istatu(4).ne.6) then                        0725YC97
      do iopc = 3,4
        if (irepr(iopc).ne.0) then
            if (istatu(iopc).ne.6) then                                 0729YC97
            if (nratom(iopc).gt.1) then
              if (ioptgr(iopc).eq.1.and.istatu(iopc).lt.2) then         0725YC97
                 call geoin(iopc,ircm)
                 if (ircm.eq.1) call rchmul(iopc)
              else
                 imultipl(iopc)=nedeg(3*iopc-2)                         0725YC97
                 do i=1,nratom(iopc)                                    0729YC97
                     inum = iatsv(i,iopc)                               0729YC97
                     do j = 2,0,-1                                      0729YC97
                        x(inum*3-j) = xr(inum*3-j,3)                    0729YC97
                     enddo                                              0729YC97
                 enddo                                                  0729YC97
              endif
                 if (isavect.ne.2) then
                 call secin(iopc)
                 endif
            else
              if (ioptgr(iopc).eq.1) then
                 call enatin(iopc,ircm)
                 if (ircm.eq.1) call rchmul(iopc)
              else
                 itmp=iop
                 iop=-iopc
                 call ehook(0)    
                 iop=itmp
              endif
            endif
            else                                                        0729YC97
              v = steng(iopc)                                           0729YC97
              do i=1,nratom(iopc)                                       0729YC97
              inum = iatsv(i,iopc)                                      0729YC97
                 do j = 2,0,-1                                          0729YC97
                    x(inum*3-j) = xr(inum*3-j,3)                        0729YC97
                 enddo                                                  0729YC97
              enddo                                                     0729YC97
            endif                                                       0729YC97
c ---       vjdep(iopc)=v
      eprod=eprod+v
        if (isup.eq.1) then
            if (iopc.eq.3) then                                         1201JC97
               eprod1=v
            else
               eprod2=v
            endif
        endif
        endif
      enddo
        if (isup.eq.0) then
            itmp=iop
            iop=5
            call ehook(0)
            iop=itmp
            eprod=v
        endif

c --- saves the result for the following calculations on these points

       enprsv=eprod
       do i=1,n3tm
       geprsv(i)=x(i)
       xr(i,3)=x(i)
       x(i)=0.d0
       fsprsv(i)=DX(i)
       dx(i)=0.d0
        do j=1,n3tm
        hsprsv(i,j)=fmat(i,j)
        fmat(i,j)=0.d0
        enddo
       enddo
      endif                                                             0725YC97

c --- Calculations for wells                                            1128JC97

      if (istatu(7).ne.6.or.istatu(8).ne.6) then                        1128JC97
       do iopc = 7,8                                                    1128JC97
        if (irepr(iopc).ne.0) then                                      1128JC97
            if (istatu(iopc).ne.6) then                                 1128JC97
              if (ioptgr(iopc).eq.1.and.istatu(iopc).lt.2) then         1128JC97
                 call geoin(iopc,ircm)                                  1128JC97
                 if (ircm.eq.1) call rchmul(iopc)                       1128JC97
              else                                                      1128JC97
                 imultipl(iopc)=nedeg(3*iopc-2)                         1128JC97
                 do i=1,nratom(iopc)                                    1128JC97
                     inum = iatsv(i,iopc)                               1128JC97
                     do j = 2,0,-1                                      1128JC97
                        x(inum*3-j) = xr(inum*3-j,iopc)                 1128JC97
                     enddo                                              1128JC97
                 enddo                                                  1128JC97
              endif                                                     1128JC97
                 if (isavect.ne.2) then                                 1128JC97
                    call secin(iopc)                                    1128JC97
                 endif                                                  1128JC97
            else                                                        1128JC97
              v = steng(iopc)                                           1128JC97
              do i=1,nratom(iopc)                                       1128JC97
              inum = iatsv(i,iopc)                                      1128JC97
                 do j = 2,0,-1                                          1128JC97
                    x(inum*3-j) = xr(inum*3-j,iopc)                     1128JC97
                 enddo                                                  1128JC97
              enddo                                                     1128JC97
            endif                                                       1128JC97
        endif                                                           1128JC97

c --- saves the result for the following calculations on these points

        if (iopc.eq.7) then                                             1128JC97
           enwrsv=V                                                     1128JC97
           do i=1,n3tm                                                  1128JC97
              gewrsv(i)=x(i)                                            1128JC97
              xr(i,7)=x(i)                                              1128JC97
              x(i)=0.d0                                                 1128JC97
              fswrsv(i)=dx(i)                                           1128JC97
              dx(i)=0.d0                                                1128JC97
              do j=1,n3tm                                               1128JC97
                hswrsv(i,j)=fmat(i,j)                                   1128JC97
                fmat(i,j)=0.d0                                          1128JC97
              enddo                                                     1128JC97
           enddo                                                        1128JC97
        else                                                            1128JC97
           enwpsv=V                                                     1128JC97
           do i=1,n3tm                                                  1128JC97
              gewpsv(i)=x(i)                                            1128JC97
              xr(i,7)=x(i)                                              1128JC97
              x(i)=0.d0                                                 1128JC97
              fswpsv(i)=dX(i)                                           1128JC97
              dx(i)=0.d0                                                1128JC97
              do j=1,n3tm                                               1128JC97
                hswpsv(i,j)=fmat(i,j)                                   1128JC97
                fmat(i,j)=0.d0                                          1128JC97
              enddo                                                     1128JC97
           enddo                                                        1128JC97
        endif                                                           1128JC97
       enddo                                                            1128JC97
      endif                                                             1128JC97

c --- Calculations for starting geometry

      iopc=5
      if (irepr(5).eq.1) then                                           0811YC97
         if (ioptgr(iopc).eq.1.and.istatu(5).lt.2) then                 0725YC97
            call geoin(iopc,ircm)
            if (ircm.eq.1) call rchmul(iopc)
         else
            imultipl(5)=nedeg(13)                                       0725YC97
                 do inatnop=1,n3tm
                 x(inatnop)=xr(inatnop,5)
                 enddo
         endif
   
c --- For avoiding problems related to reorientation, we will translate 
c     origin of coordinates to the center of masses of the optimized 
c     saddle point (energy, gradients and hessians will not change, but
c     it will be safer for the calculation of the reaction path).

           totm=0.d0
           do i=1,3
              xcm(i)=0.d0
           enddo
           do k=1,natom
             ams=amass(k*3)**2.0d0
             totm=totm+ams
             do l=1,3
               xcm(l)=xcm(l)+x((k-1)*3+l)*ams
             enddo
           enddo
           do k=1,natom
               do l=1,3
                  x((k-1)*3+l)=x((k-1)*3+l)-xcm(l)/totm
               enddo
           enddo

c --- Now, it calculates the gradients and hessian for the saddle point

           if (istatu(5).lt.4) then                                     0725YC97
                 if (isavect.ne.2) then
                 call secin(iopc)
                 endif
           endif                                                        0725YC97
                 estar=v

c --- saves the result for the following calculations on these points

       enstsv=estar
       do i=1,n3tm
       gestsv(i)=x(i)
       xr(i,5)=x(i)
       x(i)=0.d0
       fsstsv(i)=DX(i)
       dx(i)=0.d0
        do j=1,n3tm
        hsstsv(i,j)=fmat(i,j)
        fmat(i,j)=0.d0
        enddo
       enddo
      endif
      endif                                                             0811YC97

      return
1001  format (3x,'Gaussrate error: SUPERMOL keyword is only supported ',
     *'along with the',/,3x,'following options:',//,7x,'OHOOK option ',
     *'in OPTTS and OPTMIN',/,7x,'HHOOK option in HESSCAL',/,7x,'HAR',
     *'MONIC or HINROT approximation for vibrational modes')
      end                                           



c***********************************************************************
c     PREPJ
c***********************************************************************

      subroutine prepj(jtype)

c     Created by : Jingzhi Pu, July 2000
c 
c     Description :
c         This routine is the species dependent
c       driver for all preparation routines needed to set up an electronic 
c       structure calculation.  Any initializations that are dependent
c       on wheter the reactants, products, saddle point, or generalized
c       transition state is computed should be done here.  JTYPE specifies
c       the species:
c
c           JTYPE               Species
c             1                 Reactant #1
c             2                 Reactant #2
c             3                 Product #1
c             4                 Product #2
c             5                 Saddle point
c             6                 Generalized transition states
c             7                 Reactant well
c             8                 Product well
c
c       This routine is called after each respective call to stvarj.
c
c     Called by :
c     Calls : stlgsj, stvarj

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'
      common /ncgcom/ iop
      common /eshscm/ fmat(n3tm,n3tm)
      if (jtype.ne.6) then

c --- STLGSJ converts the stationary-point-dependent variables and      0729PJ00       
c     flags set by evaluating the keywords to corresponding LGS flags   0729PJ00


         call stlgsj(jtype)


c --- STVARJ converts the stationary-point-dependent variables and      0729PJ00
c     flags set by evaluating the keywords to the corresponding         0729PJ00
c     POLYRATE variables.                                               0729PJ00

         call stvarj(jtype)

      end if
      iop=jtype
      return
      end



c***********************************************************************
c     RCHMUL
c***********************************************************************

      subroutine rchmul(jtype)

c     Created by : Jingzhi Pu, July 2000
c
c     Description :
c         This subroutine reads the charge and multiplicity from a
c       Mutilevel formated Checkpoint file
c
c     Called by : prep, geoin
c     Calls : 

      implicit double precision (a-h,o-z)
 
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'

      character*80 strread
      character*80 case
      character*6  charid
      character*12 multid

c --- It starts looking for the charge by means of the title
c     'Charge'

c --- A Multilevel summary file called 'ml.sum' is created              0816PJ00

      open (unit=fu85,status='unknown',file='ml.sum')                   0816PJ00 

      charid='charge'
1015  read (fu85,1041,end=1005) strread
      do 120 ilinpos=1,3
          strread=case(strread)
          if (strread(ilinpos:ilinpos+5).ne.charid) then
                    goto 120
             else

c --- The reading of charge is start after the charid

                    is=ilinpos + 6                                      0816PJ00         
10                  if (strread(is:is).eq. ' ') then
                      is=is+1
                      goto 10
                    endif
                    icharge(jtype)=icint(strread(is:80))
                    goto 6025
          endif
120   continue
      goto 1015
1005  write (fu6,*) 'Error reading Multilevel summary file'
      stop


c --- Now, it will look for the multiplicity, by means
c     of the title 'Multiplicity'

6025  multid='multiplicity'
6015  read (fu85,1041,end=1005) strread
      do 6120 ilinpos=1,3
          strread=case(strread)
          if (strread(ilinpos:ilinpos+11).ne.multid) then
                    goto 6120
             else


c --- The reading of Multiplicity is start after the multid
c
                    is=ilinpos + 12                                     0816PJ00 
20                  if (strread(is:is).eq. ' ') then
                      is=is+1
                      goto 20
                    endif
                    imultipl(jtype)=icint(strread(is:80))
                    goto 1025
          endif
6120  continue
      goto 6015
1025  close (fu85)
      return
1041  format (a80)
      end



c***********************************************************************
c     RCODMR
c***********************************************************************

      subroutine rcodmr(icount,slinrd)                                  0811PJ00

c     Created by : Jingzhi Pu, July 2000
c     Purpose : grab every line of keyword between 'option' and 'end'   0811PJ00
c   
c     Description :
c        subroutine to read lines in every sub-section from the input   0811PJ00
c     for the routes, then the line is pushed into the array slinrd,    0811PJ00
c     the id of the line is recorded by the subscrip icount. These      0811PJ00
c     input will be used for the Multilevel runs in the future.         0811PJ00
c
c        The reading stops when the line hits the the section ending    0826PJ00
c     marker 'END'                                                      0826PJ00
c
c     Called by :  renermr, rfirstmr, rgenbas, rlincmr                  0811PJ00
c     Calls :
 
      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'
      character*80 slinrd(100)
      character*80 string, case, tempchar
      character*3 qend
      external case


c --- icount only count the number of input line between keyword        0826PJ00
c     and the 'END'                                                     0826PJ00 
 
      icount=0

c --- Read the line

1     read (fu70,'(a80)') string
      iblnk = 1

c --- Check if it is the 'end' line

      do while (iblnk.eq.1)
         i=1
         do while (iblnk.eq.1.and.i.le.80)
            if (string(i:i).ne.' ') iblnk=0
            i=i+1
         enddo
      enddo
      tempchar=string
      qend=case(tempchar(i-1:i+1))
 

c --- if it is not the end line, add the readed line to the array containing 
c     the input. Otherwise, finish reading

      if (qend.ne.'end') then
         slinrd(icount+1)=string
         icount=icount+1
         if (icount.eq.100) then
              write (fu6,*) 'Input too long. Maximum: 100 lines'
              stop
         endif
         goto 1
      else
         return
      endif

c --- So until now, the line describe the option is read to the array   0729PJ00     
c     element slinrd(icount), which is return to the caller.            0729PJ00

      return
      end



c***********************************************************************
c     REDINF
c***********************************************************************

      subroutine redinf(i,fmat,ierr)

c     Created by : Jingzhi Pu, July 2000
c     Purpose : Read information from the Gaussrate common block        0801PJ00        
c               into Polyrate.                                          0801PJ00
c     
c     Description :
c       This subroutine reads the information about reactants, products and
c       saddle point and gives that information to polyrate, avoiding the
c       recalculation of properties that were previously calculated.
c       That information was stored in the matrices that go through the
c       "savcom" common.
c       The information on wells has also been added (JC, 11/28/97)
c
c       The subroutine checks that the geometry is the same that the one it
c       has stored; otherwise, ierr=1 and Polyrate will calculate the
c       information elsewhere.
c
c     Called by : ehook, ghook
c     Calls :

      implicit double precision(a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'
      dimension fmat(n3tm,n3tm)
      dimension storcm(3),ancm(3)
      ierr=0
      alimg=1.0d-12
      do l=1,3
      storcm(l)=0.d0
      ancm(l)=0.d0
      enddo
      totm=0.d0

c --- First, check the input geometry against the stored one, measuring
c     the distance between the atoms and the center of mass in both
c     cases (this is only valid for stationary points, since along the
c     reaction coordinate, global translation or rotation will cause
c     problems). If isavect=1, then the check is made for the global
c     positions instead of the positions with respect to the center
c     of masses. If isavect=2, no check is done. This is the safer option,
c     to be used when calculating anharmonicities or numerical second
c     derivatives

      if (isavect.eq.2) then
               ierr=1
               return
      endif
      if (isavect.eq.1) then
          if (i.ne.5) then                                              1128JC97
            do j=1,natom
              do k=1,natom
                if (j.eq.iatsv(k,i)) then
                    do l=1,3
                      if (i.lt.3) then

c --- How to derive the (j-1)*3 + l here? what does the i stand for?    0816PJ00

                         dif = x((j-1)*3+l)-geresv((j-1)*3+l)
                      else
                         dif = x((j-1)*3+l)-geprsv((j-1)*3+l)
                      endif

c --- alimg is some small number, here is set to 1.0d-12 
c     so use alimg to check wether the difference is small enough       0816PJ00


                      if (abs(dif).gt.alimg) then
                         ierr=1
                         return
                      endif
                    enddo
                 endif
              enddo
            enddo
          else
            do j=1,natom
                    do l=1,3
                         dif = x((j-1)*3+l)-gestsv((j-1)*3+l)
                      if (abs(dif).gt.alimg) then
                         ierr=1
                         return
                      endif
                    enddo
            enddo
          endif
       endif
      if (i.ne.5) then                                                  1128JC97
          do 1 j=1,natom
           do 2 k=1,natom
             if (j.eq.iatsv(k,i)) then
                 ams=amass(j*3)**2.0D0
                 totm=totm+ams
                do l=1,3

c --- Calculate the center of mass of the current position (ancm) and of
c     the stored positions (storcm).

                 ancm(l)=ancm(l)+x((j-1)*3+l)*ams
                 if (i.lt.3) then
                  storcm(l)=storcm(l)+geresv((j-1)*3+l)*ams
                 else if (i.le.4) then                                  1128JC97
                  storcm(l)=storcm(l)+geprsv((j-1)*3+l)*ams
                 else if (i.eq.7) then                                  1128JC97
                  storcm(l)=storcm(l)+gewrsv((j-1)*3+l)*ams             1128JC97
                 else if (i.eq.8) then                                  1128JC97
                  storcm(l)=storcm(l)+gewpsv((j-1)*3+l)*ams             1128JC97
                 endif
                enddo
             endif
2          continue
1         continue
          do 3 j=1,natom
           do 4 k=1,natom
             if (j.eq.iatsv(k,i)) then
                dif=0.d0
                disnx=0.d0
                dissx=0.d0
                do l=1,3


c --- Calculate the distance between the atoms and the center of mass
c     in both cases and compare them

                  disnx=disnx+(x((j-1)*3+l)-ancm(l)/totm)**2.0D0
                  if (i.lt.3) then
                   dissx=dissx+(geresv((j-1)*3+l)-storcm(l)/totm)**2.0D0
                  else if (i.le.4) then                                 1128JC97
                   dissx=dissx+(geprsv((j-1)*3+l)-storcm(l)/totm)**2.0D0
                  else if (i.eq.7) then                                 1128JC97
                   dissx=dissx+(gewrsv((j-1)*3+l)-storcm(l)/totm)**2.0D01128JC97
                  else if (i.eq.8) then                                 1128JC97
                   dissx=dissx+(gewpsv((j-1)*3+l)-storcm(l)/totm)**2.0D01128JC97
                  endif
                enddo
                  dif=sqrt(disnx)-sqrt(dissx)
                    if (abs(dif).gt.alimg) then
                        ierr=1
                        return
                    endif
             endif
4          continue
3         continue
      endif

      if (i.eq.5) then                                                  1128JC97


c --- We have to do the same for the saddle point, although a bit 
c     simpler, since all the atoms are involved.

          do 5 k=1,natom
            ams=amass(k*3)**2.0D0
            totm=totm+ams
            do l=1,3


c --- Calculate the center of mass of the current position (ancm) 
c     and of the stored positons (storcm).

               ancm(l)=ancm(l)+x((k-1)*3+l)*ams
               storcm(l)=storcm(l)+gestsv((k-1)*3+l)*ams
            enddo
5        continue
                dif=0.d0
                disnx=0.d0
                dissx=0.d0
          do 6 k=1,natom
            do l=1,3


c --- Calculate the distance between the atoms and the center of 
c     mass in both cases and compare them

                 disnx=disnx+(x((k-1)*3+l)-ancm(l)/totm)**2.0D0
                 dissx=dissx+(gestsv((k-1)*3+l)-storcm(l)/totm)**2.0D0  
            enddo
                 dif=sqrt(disnx)-sqrt(dissx)
                 if (abs(dif).gt.alimg) then
                        ierr=1
                        return
                 endif
6         continue
      endif

c --- If the program is still in this subroutine, it is because the 
c     comparison of geometries indicated that we are at the stored
c     point. Therefore, now it has to read the saved information 

      if (i.le.2) then
            v=enresv
            if (isup.eq.1) then
                if (i.eq.1) v=ereac1
                if (i.eq.2) v=ereac2
            endif
            do k=1,n3tm
            x(k)=geresv(k)
            dx(k)=fsresv(k)
               do l=1,n3tm
                   fmat(k,l)=hsresv(k,l)
               enddo
            enddo
      else if (i.le.4) then
            v=enprsv
            if (isup.eq.1) then
                if (i.eq.3) v=eprod1
                if (i.eq.4) v=eprod2
            endif
            do k=1,n3tm
            x(k)=geprsv(k)
            dx(k)=fsprsv(k)
               do l=1,n3tm
                 fmat(k,l)=hsprsv(k,l)
               enddo
            enddo
      else if (i.eq.5) then                                             1128JC97
            v=enstsv
            do k=1,n3tm
            x(k)=gestsv(k)
            dx(k)=fsstsv(k)
               do l=1,n3tm
                 fmat(k,l)=hsstsv(k,l)
               enddo
            enddo
      else if (i.eq.7) then                                             1128JC97
            v=enwrsv                                                    1128JC97
            do k=1,n3tm                                                 1128JC97
            x(k)=gewrsv(k)                                              1128JC97
            dx(k)=fswrsv(k)                                             1128JC97
               do l=1,n3tm                                              1128JC97
                 fmat(k,l)=hswrsv(k,l)                                  1128JC97
               enddo                                                    1128JC97
            enddo                                                       1128JC97
      else if (i.eq.8) then                                             1128JC97
            v=enwpsv                                                    1128JC97
            do k=1,n3tm                                                 1128JC97
            x(k)=gewpsv(k)                                              1128JC97
            dx(k)=fswpsv(k)                                             1128JC97
               do l=1,n3tm                                              1128JC97
                 fmat(k,l)=hswpsv(k,l)                                  1128JC97
               enddo                                                    1128JC97
            enddo                                                       1128JC97
      else                                                              1128JC97
            write (fu6,*) 'Something strange happened in REDINF. It',   1128JC97
     *                    ' entered with jtype=', i                     1128JC97
      endif
      return
      end



c***********************************************************************
c     RENERMR
c***********************************************************************

      subroutine renermr(string,istrt,jtype)                            0811PJ00

c     Created by : Jingzhi Pu, July 2000
c     Purpose : Read the input option in fu70 for Multilevel energy     0801PJ00
c               calculation.                                            0801PJ00      
c    
c     Description :
c          subroutine to read in the input for the route for the
c       energy calculations
c
c     Called by : rstatmr                                               0811PJ00  
c     Calls : rcodmr                                                    0811PJ00

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'
      character*80 slinrd(100)
      character * 80 string, case
      character * 3 qend
      external case

      call rcodmr(icount,slinrd)

c --- Jtype = 9 for all stationary points and  points along the         0811PJ00 
c     reacton path.                                                     0811PJ00

      if (jtype.ne.9) then                                              1128JC97
         inlene(jtype)=icount
         do i=1,icount
            enerot(jtype,i)=slinrd(i)
         enddo
      else
         do j=1,8                                                       1128JC97
         inlene(j)=icount
           do i=1,icount
              enerot(j,i)=slinrd(i)
           enddo
         enddo
      endif

      return
      end



c***********************************************************************
c     RFIRSTMR
c***********************************************************************

      subroutine rfirstmr(string,istrt,jtype)                           0811PJ00

c     Created by : Jingzhi Pu, July 2000
c     Purpose : read the input option for Multilevel gradient           0729PJ00          
c               calculation                                             0729PJ00
c 
c     Description :
c          subroutine to read in the input for the route for the
c        first derivative calculations
c
c          Almost as same as RENERGR, maybe first call RENERGR, then    0729PJ00
c        call RFIRSTGR, the sequence is determined by RESTATGR          0729PJ00
c
c     Called by : rstatmr                                               0811PJ00
c     Calls : rcodmr,                                                   0811PJ00 

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'
      character*80 slinrd(100)
      character * 80 string, case
      character * 3 qend
      external case

       call rcodmr(icount,slinrd)
       if (jtype.ne.9) then                                             1128JC97
           inlfrs(jtype)=icount
           do i=1,icount
                  firrot(jtype,i)=slinrd(i)
           enddo
       else
          do j=1,8                                                      1128JC97
          inlfrs(j)=icount
            do i=1,icount
               firrot(j,i)=slinrd(i)
            enddo
          enddo
       endif

      return
      end



c***********************************************************************
c     RGENMR
c***********************************************************************

      subroutine rgenmr(string,iend,istrt)

c     Created by : Jingzhi Pu, July 2000
c     Purpose : read the option in the general section of fu70 input    0729PJ00       
c               file.                                                   0729PJ00
c
c     Description :
c          subroutine to read in the input in the general section. By
c        now, it only includes the restart option. Other sections 
c        apart from mrgeneral section are read by 
c        Multirate .70 file are re
c
c     Called by : rinpgr,
c     Calls : rlinmr, 

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'
      character * 80 string


c --- set the defaults:


c --- GRRESTART: ON

      irest = 1
  
c --- MTLV: ON                                                          1026PJ00 
    
      mtlv = 1                                                          1026PJ00 


c --- read in first keyword in this section

      call rlinmr(string,istrt,isect,iend)

      do while (isect.eq.0.and.iend.eq.0)


c --- MRRESTART                                                         0812PJ00  
     
         if (string(istrt:istrt+4) .eq. 'mrres') then                   0812PJ00
            irest = 1

c --- NOMRRESTART                                                       0812PJ00

         else if (string(istrt:istrt+6) .eq. 'nomrres') then            0812PJ00
            irest = 0

c --- SLLV                                                              1026PJ00         

         else if (string(istrt:istrt+3) .eq. 'sglv') then               1026PJ00     
            mtlv = 0 

c --- MTLV                                                              1026PJ00

         else if  (string(istrt:istrt+3) .eq. 'mtlv') then              1026PJ00
            mtlv = 1                                                    1026PJ00

c --- INVALID KEYWORD

         else
            write(fu6,1200) string(istrt:80)
            stop
         endif
 

         call rlinmr(string,istrt,isect,iend)

      end do

1200  format(3x,'Error:  the following is not a valid keyword in the',
     *       ' grgeneral section',/A80)

      return
      end



c***********************************************************************
c     RINPMR
c***********************************************************************

      subroutine rinpmr

c     Created by : Jingzhi Pu, July 2000
c
c     Description : read the gaussian input file fu70
c 
c     Called by : setup 
c     Calls : rlinmr, rgenmr, rstatmr                                   0811PJ00

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'
      character * 80 string,case
      inlcer=0
      inlbas=0
      do i=1,8                                                          1128JC97
       inlscr(i)=1
       inlfrs(i)=1
       inlene(i)=1

c --- set the default theory of level and basis set, etc. key words     0729PJ00 
c     for Gaissian calculation                                          0729PJ00

       secrot(i,1) = 'SAC-MP2/pDZ ENERGY GRADIENT HESSIAN'              0812PJ00
       firrot(i,1) = 'SAC-MP2/pDZ ENERGY GRADIENT NOHESSIAN'            0812PJ00
       enerot(i,1) = 'SAC-MP2/pDZ ENERGY NOGRADIENT NOHESSIAN'          0812PJ00
      enddo

c --- This subroutine reads the input of the Multirate options. It
c     starts reading if it is a Grate restart run. If it is, open
c     the reading file and the writing file.

      call rlinmr(string,istrt,isect,iend)


c --- Those error check lines are equivalents to those when reading fu5

      if (iend.eq.1) then
         write(fu6,1000)
1000     format(1x,'error:  input file was empty')
         stop
      end if

      if (isect.ne.1) then
         write(fu6,1100)
1100     format(1x,'error:  first non-comment, non-blank line must be'
     *             ,' a section header that begins with a *')
         stop
      end if


c --- Now, it reads the sections in Multirate input

      do while (iend.eq.0)
         j = istrt + 1
         if (string(j:j+8).eq.'mrgeneral') then                         0811PJ00      
               call rgenmr(string,iend,istrt)    
         else if (string(j:j+7).eq.'mrreact1') then                     0811PJ00
               call rstatmr(string,iend,istrt,1)
         else if (string(j:j+7).eq.'mrreact2') then                     0811PJ00
               call rstatmr(string,iend,istrt,2)
         else if (string(j:j+6).eq.'mrprod1') then                      0811PJ00
               call rstatmr(string,iend,istrt,3)
         else if (string(j:j+6).eq.'mrprod2') then                      0811PJ00
               call rstatmr(string,iend,istrt,4) 
         else if (string(j:j+6).eq.'mrstart') then                      0811PJ00 
               call rstatmr(string,iend,istrt,5)
         else if (string(j:j+5).eq.'mrpath') then                       0811PJ00
               call rstatmr(string,iend,istrt,6)
         else if (string(j:j+6).eq.'mrwellr') then                      0811PJ00
               call rstatmr(string,iend,istrt,7)                        
         else if (string(j:j+6).eq.'mrwellp') then                      0811PJ00
               call rstatmr(string,iend,istrt,8)                       
         else if (string(j:j+7).eq.'mrcommon') then                     0811PJ00
               call rstatmr(string,iend,istrt,9)                      
         else
               write(fu6,1200) string(j-1:80)
               stop
         end if
      end do

1200  format(3x,'Error:  the following is not a valid section name',
     *         /a80)


c --- if the restart is used in the general section, open the restart   0816PJ00
c     related file fu83 and fu84                                        0816PJ00

      if (irest.eq.1) then
         open (unit=fu83,file='esp.fu83')
         open (unit=fu84,status='new',file='esp.fu84')
         write(fu6,2001)
      endif

2001     format(/,5x,'Mutilevel restart calculation',/)

      return
      end




c***********************************************************************
c     RLINMR
c***********************************************************************

      subroutine rlinmr(string,istrt,isect,iend)                        0811PJ00

c     Created by : Jingzhi Pu, July 2000
c     Purpose : read a line in input fu70 and parsing off the comments  0729PJ00          
c               and extra spaces.                                       0729PJ00
c
c     Description :
c        This subroutine is equivalent to rline in interface.f, but a bit
c     modified for reading from unit fu70
c
c     iblnk:   the marker to identify the line's blank status           0803PJ00  
c              iblnk = 1, blank line, continue read nex line            0803PJ00
c              iblnk = 0, none blank, return                            0803PJ00
c
c     istrt:   location in 80 character string that is the first non-blank
c     isect:   a 1 is returned if the first non-blank character is a *
c              signalling the start of a section
c     iend:    a 1 is returned if the end of file 5 is encountered
c     string:  80 character string that is the contents of the current line
c
c     Called by : rgenmr, rstatmr, 
c     Calls :  

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'

      character * 80 string,case
      external case

1000  format(a80)

c --- initialize variables

      isect = 0
      iend = 0
      iblnk = 1

c --- read in next line in file 5 - find first non-blank character and
c     determine if it is a comment (# sign signals a comment).  Once a
c     comment is found the rest of the line is skipped (by setting the
c     counter to the last character [i=80])

      do while (iblnk.eq.1)
         read(fu70,1000,end=9999) string
         i = 1
         do while (iblnk.eq.1.and.i.le.80)
            if (string(i:i).ne.' ') then
               if (string(i:i).ne.'#') then
                   iblnk = 0
                else
                   i = 80
                end if
            end if
            i = i+1
         end do
       end do


c --- set location of first non-blank character

       istrt = i-1


c --- check if it is a section header (first character a *)

       if (string(istrt:istrt).eq.'*') isect=1


c --- strip of any trailing comments on the line

       do 1 j=istrt,80
1        if (string(j:j).eq.'#') string(j:80) = ' '


c --- change to lower case

       string = case(string)
       return
9999   iend=1
       return
       end



c***********************************************************************
c     RSECMR
c***********************************************************************

      subroutine rsecmr(string,istrt,jtype)                             0811PJ00
   
c     Created by : Jingzhi Pu, July 2000
c     Purpose : read fu70 for second derivative calculation for         0801PJ00   
c               Multilevel.                                             0801PJ00
c 
c     Description :
c        subroutine to read in the input for the route for the
c      second derivative calculations
c
c     Called by : rstatmr,                                              0811PJ00
c     Calls : rcodmr                                                    0811PJ00 

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'
      character*80 slinrd(100)
      character * 80 string, case
      character * 3 qend
      external case

      call rcodmr(icount,slinrd)                                        0811PJ00
      if (jtype.ne.9) then                                              1128JC97
         inlscr(jtype)=icount
         do i=1,icount
            secrot(jtype,i)=slinrd(i)
         enddo
       else
          do j=1,8                                                      1128JC97
          inlscr(j)=icount
            do i=1,icount
               secrot(j,i)=slinrd(i)
            enddo
          enddo
       endif

      return
      end



c***********************************************************************
c     RSTATMR
c***********************************************************************

      subroutine rstatmr(string,iend,istrt,jtype)                       0811PJ00
   
c     Created by : Jingzhi Pu, July 2000
c     Purpose : read the sections in fu70 file.                         0811PJ00
c 
c     Description:
c         subroutine to read in the section informations in fu70 file,  0811PJ00
c       work for all sections except MRGENERAL section, which is dealt  0811PJ00
c       with anther subroutin RGENMR to setup the restart process.      0811PJ00
c
c     Comment: rword is a subroutine in interface.f                     0826PJ00
c              it returns the word's last position in istrt,            0826PJ00
c              if there's no other word in the line, ierr return        0826PJ00 
c              value of 1.                                              0826PJ00 
c 
c     Called by : rinpmr                                                0811PJ00
c     Calls : rlinmr, rgenbas, renermr, rfirstmr, rsecmr, rword         0811PJ00

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'
      character * 80 string

c --- JTYPE 's value and significance
c
c     Jtype = 1  first reactant
c     Jtype = 2  second reactant
c     Jtype = 3  first product
c     Jtype = 4  second product
c     Jtype = 5  saddle point or starting stationary point
c     Jtype = 6  all the points along the path
c     Jtype = 7  reactant well
c     Jtype = 8  product well
c     Jtype = 9  all the stationary points and points along the path

c --- read in first keyword in this section

      call rlinmr(string,istrt,isect,iend)

      do while (isect.eq.0.and.iend.eq.0)


c --- MROPT                                                             0811PJ00
  
         if (string(istrt:istrt+4).eq.'mropt') then                     0811PJ00
            write (fu6,1150)                                            1128JC97
            stop                                                        1128JC97


c --- NOMROPT                                                           0811PJ00

         else if (string(istrt:istrt+6).eq.'nomropt') then              0811PJ00  
            write (fu6,1150)                                            1128JC97
            stop                                                        1128JC97
 

c --- NOATOMS

         else if (string(istrt:istrt+6).eq.'noatoms') then              0811PJ00
            call rword(string,istrt,ierr)                               0811PJ00 
            if(ierr.ne.1) then                                          0811PJ00
              if (jtype.ne.9.and.jtype.ne.6) then                       1128JC97
               icharge(jtype) = icint(string(istrt:80))                 0811PJ00
              else                                                      0811PJ00    
               write(fu6,1050)                                          0811PJ00
               stop                                                     0811PJ00
              endif                                                     0811PJ00
            else                                                        0811PJ00
               write(fu6,1100)                                          0811PJ00  
               stop                                                     0811PJ00 
            end if                                                      0811PJ00 
 

c --- CHARGE

         else if (string(istrt:istrt+5).eq.'charge') then
            call rword(string,istrt,ierr)
            if(ierr.ne.1) then
              if (jtype.ne.9.and.jtype.ne.6) then                       1128JC97
               icharge(jtype) = icint(string(istrt:80))
              else 
               write(fu6,1050)
               stop
              endif
            else
               write(fu6,1100)
               stop
            end if


c --- MULTIPLICITY
 
         else if (string(istrt:istrt+5).eq.'multip') then
            call rword(string,istrt,ierr)
            if(ierr.ne.1) then
              if (jtype.ne.9.and.jtype.ne.6) then                       1128JC97
                 imultipl(jtype) = icint(string(istrt:80))
              else 
                 write(fu6,1050)
                 stop
              endif
            else
               write(fu6,1100)
               stop
            end if


c --- GRLINK0 and GRBASIS are removed for Multirate                     0826PJ00



c --- MRENER                                                            0811PJ00

         else if (string(istrt:istrt+5).eq.'mrener') then               0811PJ00
            call renermr(string,istrt,jtype)                            0811PJ00


c --- MRFIRST                                                           0811PJ00

         else if (string(istrt:istrt+6).eq.'mrfirst') then              0811PJ00
            call rfirstmr(string,istrt,jtype)                           0811PJ00


c --- MRSEC                                                             0811PJ00

         else if (string(istrt:istrt+4).eq.'mrsec') then                0811PJ00
            call rsecmr(string,istrt,jtype)                             0811PJ00
         else
            write(fu6,1200) string(istrt:80)
            stop
         end if
         call rlinmr(string,istrt,isect,iend)                           0811PJ00
      end do

1050  format(3x,'Error:  NOATOMS,MULTIPLICITY and CHARGE can',          0103PJ04
     *       ' not be in COMMON or PATH sections',/)
1100  format(3x,'Error reading NOATOMS, CHARGE or MULTIPLICITY',/)      0811PJ00
1150  format(3x,'Error:  MROPT and NOMROPT are not longer valid',       0811PJ00  
     *       ' keywords',/)
1200  format(3x,'Error:  the following is not a valid keyword in',
     *       ' this section',/A80)

      return
      end



c***********************************************************************
c     HHOOK
c***********************************************************************

      subroutine hhook(xii,fmat,sstp,nend,mw,icent)

c     Created by : Jingzhi Pu, July 2000
c 
c     Description :
c
c     This routine optionally accepts the mass-weighted cartesian 
c     coordinates and transforms them to cartesian coordinates (mw=1).
c     The cartesian coordinates are used when computing the energy and
c     second derivatives of the energy with respect to each cartesian 
c     coordinate.  If the mass-weighted option was chosen (mw=1), the 
c     coordinates and first derivatives are transformed back to mass-
c     weighted at the end.  
c
c     Called by :
c     Calls : trans, seccen, seccep, derive2, derv24, mrest, redinf,
c             escsec, effbath

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'
      common /ncgcom/ iop
      dimension xii(n3tm), fmat(n3tm,n3tm), tmpfmat(n3tm,n3tm)
      dimension bup(n3tm)
     

c --- Optionally convert the mass-weighted coordinates to cartesian

      if (mw.eq.1) call trans(2,n3,amass,x,dx)

c --- If the hessian is calculated using polyrate:


c --- In this case, no need to modify                                   0729PJ00      

      if (ispot.eq.0) then
          if (lgs(25).eq.0) then
             if (icent.eq.0) then
                 do 75 i=1,n3
75                 bup(i) = x(i)
                 call seccen (bup,fmat,mw,sstp)
                 goto 1
             endif
             if (icent.eq.1) then
                 call seccep (mw,sstp,nend)
                 goto 1
             endif
          else if (lgs(25).eq.1) then
                 call deriv2(nend)
                 goto 1
          else if (lgs(25).eq.2) then
                 call derv24(nend)
                 goto 1
          endif
       endif

c --- If it is calculated using Gaussian:
c     If it is a Grate restart, read the energy, 1st and 2nd der.
     
      irout=3
      if (irest.eq.1) call mrest(irout,fmat)
      if (irout.eq.0) return


c --- If the calculation corresponds to a stationary point, check if it has
c     already been calculated and stored in redinf.

        if (iop.eq.6) then                                              1128JC97
            i=5
            nacat=natom
        else 
            if (iop.eq.5) then
                nacat=natom
            else
                nacat=nratom(iop)
            endif
            i=iop
            call redinf(iop,fmat,ierr)
            if (ierr.eq.0) goto 149 
        endif
               ich=icharge(i)
               imul=imultipl(i)
               do 119 indfmex=1,n3tm
                do 129 indfmey=1,n3tm
                 fmat(indfmex,indfmey)=0.d0
129             continue
119            continue
               call escsec(natoms,ich,imul,natom,label,
     *                     iatsv,v,x,dx,fmat,iop,nacat)

c --- Reorder the stored hessian (if we are dealing with reactants or
c     products) and convert into mass-weighted

149       if (iop.ne.5.and.iop.ne.6) then                               1128JC97
            do l=1,n3tm
            do m=1,n3tm
                tmpfmat(l,m)=fmat(l,m)
                fmat(l,m)=0.0D0
            enddo
            enddo
            do l=1,nratom(iop)
            do m=1,nratom(iop)
                  do k=1,3
                  do n=1,3
                     innx=3*(iatsv(l,iop)-1)+k
                     inny=3*(iatsv(m,iop)-1)+n
                     inox=3*(l-1)+k
                     inoy=3*(m-1)+n
                          fmat(inox,inoy)=tmpfmat(innx,inny)/
     *                     amass(ind(inox))/amass(ind(inoy))
                  enddo
                  enddo
              enddo
              enddo
           else

c --- If it is the saddle point, we only have to convert the Hessian
c     into mass-weighted

c --- for non-equilibrium solvation, need to add an extra harmonic term

            nend = natom*3

            if (lbath.and.lfopt) then
              nend = nend + 1
              f1 = pi*pi*redm/(16.0d0*frict)
              f2 = f1/frict
              do i = 1,nend-1
                 fmat(nend,i) = -f2*diffu(i)
                 fmat(i,nend) = fmat(nend,i)
              enddo
              fmat(nend,nend)=f2
              do i = 1,nend-1
               do j = 1,i
                 fmat(i,j) = fmat(i,j)+f2*diffu(i)*diffu(j)
                 fmat(j,i) = fmat(i,j)
               enddo
              enddo
            endif
c ---          do l=1,natom*3
c ---          do m=1,natom*3
              do l=1,nend
              do m=1,nend
                fmat(l,m)=fmat(l,m)/amass(ind(l))/amass(ind(m))
              enddo
              enddo
           endif


c --- Put the appropriate zero of energy

      if (lezer0) v = v-ezer0   


c --- Calculate the derivative and energy, should be in Cartesian coord

      if (lbath) then
        call effbath(2)
      endif

 

c --- Transfer back to mass-weighted coordinates

1     if (mw.eq.1) call trans(1,n3,amass,x,dx)


c --- If it is a Grate restart, write the energy, 1st and 2nd der.
c     If ispot=0, the calculations of first derivatives or/and energies were 
c     already stored and we don't need to save the information about fmat.


      if (ispot.ne.0) then
        if (irest.eq.1) then
          irout=7
          call mrest(irout,fmat)
        endif
      endif

      return
      end                                           



c***********************************************************************
c     SETUP
c***********************************************************************

      subroutine setup

c     Created by : Jingzhi Pu, July 2000
c     Purpose : initializes some varialbes and calls the the            0729PJ00          
c               subroutines for reading in the fu70 (.70)               0729PJ00
c               input file.                                             0729PJ00
c   
c     Description :
c        This subroutine writes a title for the polyrate long output and
c      call the subroutine for reading Gaussrate input
c
c     Called by : prep,
c     Calls : rinpmr 

      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'multirate.inc'

      open (unit=fu70,status='old',file='esp.fu70')
      call rinpmr
      close (fu70)

         write (fu6,100) 

100   format (/,2x,T5,'Multilevel 3.1 will carry out the electronic'
     *        /,2x,T5,'structure calculations needed for obtaining the'
     *        /,2x,T5,'required data for the POLYRATE calculation',/)


c --- Check compatibility of options and set gropt

      if (isup.eq.0) then
         write (fu6,200)
         stop 'setup 1'
      endif


c --- OHOOK optimization:

      if (igpot.eq.1) then                                              1203JC97
         do 10 i=1,8                                                    1203JC97
           if (i.eq.6) goto 10                                          1203JC97
           if (irepr(i).eq.0) goto 10                                   1203JC97


c --- check all the systems to be optimized have initgeo=hooks
c     and set ioptgr=1

           if (istatu(i).eq.0) then                                     1203JC97
              if (initg(i).eq.1) then                                   1203JC97
                  ioptgr(i) = 1                                         1203JC97
              else                                                      1203JC97
                  write (fu6,300) i                                     1203JC97
                  stop 'setup 2'                                        1203JC97
              endif                                                     1203JC97
            else                                                        1203JC97


c --- check all the systems not to be optimized have initgeo=geom 
c     and set ioptgr=0

              if (initg(i).eq.0) then                                   1203JC97
                  ioptgr(i) = 0                                         1203JC97
              else                                                      1203JC97
                  write (fu6,400) i                                     1203JC97
                  stop 'setup 3'                                        1203JC97
              endif                                                     1203JC97
            endif                                                       1203JC97
10       continue                                                       1203JC97

 
c --- BFGS or NH optimizations

      else                                                              1203JC97
         do 20 i=1,8                                                    1203JC97
           if (i.eq.6) goto 20                                          1203JC97
           if (irepr(i).eq.0) goto 20                                   1203JC97


c --- check all the systems have initgeo=geom 

           if (initg(i).eq.0) then                                      1203JC97
                  ioptgr(i) = 0                                         1203JC97
           else                                                         1203JC97
                  write (fu6,500) i                                     1203JC97
                  stop 'setup 3'                                        1203JC97
           endif                                                        1203JC97
20       continue                                                       1203JC97
      endif                                                             1203JC97


200   format (/,2x,t5,'Error: Multirate requires the no-supermolecule'  0809PJ00
     *        /,2x,t5,'mode. The program will stop',/)                  0809PJ00  
300   format (/,2x,t5,'Error: OHOOK has been selected for the '         1203JC97
     *        /,2x,t5,'optimization, and the species ', I3,' is going'  1203JC97
     *        /,2x,t5,'to be optimized, so the geometry should be'      1203JC97
     *        /,2x,t5,'given in HOOKS (INITGEO=HOOKS).'/)               1203JC97   
400   format (/,2x,t5,'Error: OHOOK has been selected for the '         1203JC97   
     *        /,2x,t5,'optimization, and the species ', I3,' is not '   1203JC97   
     *        /,2x,t5,'going to be optimized, so the geometry should '  1203JC97     
     *        /,2x,t5,'be given in FU5 (INITGEO=GEOM).'/)               1203JC97 
500   format (/,2x,t5,'Error: BFGS or NR has been selected for the '    1203JC97   
     *        /,2X,t5,'optimization, so for the species ', I3,' the'    1203JC97   
     *       /,2x,t5,'geometry should be given in FU5 (INITGEO=GEOM).'/)1203JC97 

      return
      end



c***********************************************************************
c    STORE
c***********************************************************************

      subroutine store(v,coord,dx,fmat,indop)

c     Created by : Jingzhi Pu, July 2000
c 
c     Description :
c         This subroutine checks if the first derivative calculation is going 
c       to be carried out on a point where the second derivative has been
c       calculated right before this calcularion. If it is, it reads the
c       first derivative instead of calculating it, so that computing time
c       is saved.
c
c     Called by : escfirst
c     Calls :
 
      implicit double precision (a-h,o-z)

      include 'param.inc'
      include 'percon.inc'
      include 'multirate.inc'
      common /strcom/ storev,storecoord(n3tm),storedx(n3tm),
     *                storefmat(n3tm,n3tm)
      dimension coord(n3tm),dx(n3tm),fmat(n3tm,n3tm)
      difall=1.0D-10
      if (indop.eq.1) then
         storev=v
         do instr=1,n3tm
            storecoord(instr)=coord(instr)
            storedx(instr)=dx(instr)
              do jnstr=1,n3tm
               storefmat(instr,jnstr)=fmat(instr,jnstr)
              enddo
         enddo
      else
        do incont=1,n3tm
             dif=coord(incont)-storecoord(incont)
             if (abs(dif).gt.difall) goto 101
        enddo
        v=storev
        do incont=1,n3tm
           dx(incont)=storedx(incont)
        enddo
         indop=1
      endif
101   return
      end
c
